package com.ippon.article.clientmobile.database;

import android.content.Context;
import android.support.annotation.Nullable;
import android.util.Log;

import com.couchbase.lite.BasicAuthenticator;
import com.couchbase.lite.CouchbaseLiteException;
import com.couchbase.lite.DataSource;
import com.couchbase.lite.Database;
import com.couchbase.lite.DatabaseConfiguration;
import com.couchbase.lite.Document;
import com.couchbase.lite.Endpoint;
import com.couchbase.lite.Expression;
import com.couchbase.lite.LogDomain;
import com.couchbase.lite.LogLevel;
import com.couchbase.lite.MutableDocument;
import com.couchbase.lite.Query;
import com.couchbase.lite.QueryBuilder;
import com.couchbase.lite.Replicator;
import com.couchbase.lite.ReplicatorChange;
import com.couchbase.lite.ReplicatorChangeListener;
import com.couchbase.lite.ReplicatorConfiguration;
import com.couchbase.lite.Result;
import com.couchbase.lite.ResultSet;
import com.couchbase.lite.SelectResult;
import com.couchbase.lite.URLEndpoint;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

/**
 * Created by jeremie on 04/05/18.
 */

public class CouchbaseLite {

    private final static String TAG = "CouchbaseLite";


    @Nullable
    public static Database openDatabase(Context androidContext) throws CouchbaseLiteException {
        // Get the database (and create it if it doesn’t exist).
        DatabaseConfiguration config = new DatabaseConfiguration(androidContext);
        return new Database("localdb", config);
    }

    public static void createDocument(Database database) throws CouchbaseLiteException {
        // Create a new document (i.e. a record) in the database.
        MutableDocument mutableDoc = new MutableDocument()
                .setFloat("version", 2.0F)
                .setString("type", "SDK");

        // Save it to the database.
        database.save(mutableDoc);

        // Update a document.
        mutableDoc = database.getDocument(mutableDoc.getId()).toMutable();
        mutableDoc.setString("language", "Java");
        try {
            database.save(mutableDoc);
        } catch (CouchbaseLiteException e) {
            e.printStackTrace();
        }
        Document document = database.getDocument(mutableDoc.getId());
        // Log the document ID (generated by the database) and properties
        Log.i(TAG, "Document ID :: " + document.getId());
        Log.i(TAG, "Learning " + document.getString("language"));
    }

    public static List<Result> findAllDocuments(Database database, String javaClass) throws CouchbaseLiteException {
        // Create a query to fetch documents of type SDK.
        Query query = QueryBuilder.select(SelectResult.property("content"))
                .from(DataSource.database(database))
                .where(Expression.property("javaClass").equalTo(Expression.string(javaClass)));

        ResultSet result = query.execute();
        List<Result> resultsList = result.allResults();
        Log.i(TAG, "Number of rows ::  " + resultsList.size());
        return resultsList;
    }

    public static void configureReplication(Database database, String gatewayUrl, String gatewayUser, String gatewayPwd) throws CouchbaseLiteException, URISyntaxException {
        Database.setLogLevel(LogDomain.REPLICATOR, LogLevel.VERBOSE);
        Database.setLogLevel(LogDomain.NETWORK, LogLevel.VERBOSE);

        // Create replicators to push and pull changes to and from the cloud.
        Endpoint targetEndpoint = new URLEndpoint(new URI(gatewayUrl));
        ReplicatorConfiguration replConfig = new ReplicatorConfiguration(database, targetEndpoint);

        replConfig.setReplicatorType(ReplicatorConfiguration.ReplicatorType.PULL);

        // Add authentication.
        replConfig.setAuthenticator(new BasicAuthenticator(gatewayUser, gatewayPwd));
        replConfig.setContinuous(true);

        // Create replicator.
        Replicator replicator = new Replicator(replConfig);

        // Listen to replicator change events.
        replicator.addChangeListener(new ReplicatorChangeListener() {
            @Override
            public void changed(ReplicatorChange change) {
                if (change.getStatus().getError() != null) {
                    Log.i(TAG, "Error code ::  " + change.getStatus().getError().getCode());
                }
            }
        });

        // Start replication.
        replicator.start();
    }

}
